from PyPDF2 import PdfReader, PdfWriter
import re  # 导入正则表达式库
import os  # 用于文件和文件夹操作

def extract_common_header(pdf_path):
    pdf_document = PdfReader(pdf_path)
    headers = []
    num_pages_to_read = 3  # 预读页数

    for i in range(min(num_pages_to_read, len(pdf_document.pages))):
        page = pdf_document.pages[i]
        text = page.extract_text()
        if text:  # 确保页面有文本内容
            first_line = text.strip().split('\n')[0]
            headers.append(first_line)

    if len(headers) < 2:
        return ""  # 如果没有足够的页来比较，返回空字符串

    # 使用set交集来找出公共部分
    common_header = set(headers[0].split()).intersection(*[set(header.split()) for header in headers[1:]])
    common_header = ' '.join(common_header)
    return common_header


def clean_page_content(text, common_header):
    # 首先删除抬头公共部分
    if common_header:  # 确保有公共抬头才进行替换
        cleaned_text = text.replace(common_header, '', 1)  # 假设抬头出现在文本开头，只替换一次
    else:
        cleaned_text = text

    # 删除页码
    cleaned_text = re.sub(r'^\s*\d+\s*(?=\D)', '', cleaned_text)  # 删除开头的页码，仅当紧跟非数字字符时
    cleaned_text = re.sub(r'\s+\d+\s*$', '', cleaned_text)  # 删除结尾的页码
    cleaned_text = re.sub(r'\s*\/\s*\d+\s*', '', cleaned_text)  # 删除形如 /129 的页码

    return cleaned_text


def save_pages_to_new_pdf(pdf_path, output_folder, output_suffix, start_page, end_page):
    """从原始PDF截取指定范围的页面并保存到新的PDF文件中"""
    # 获取文件基本名称
    base_file_name = os.path.splitext(os.path.basename(pdf_path))[0]
    # 构建输出文件路径
    output_pdf_path = os.path.join(output_folder, f"{base_file_name}_{output_suffix}.pdf")

    # 读取PDF文件
    pdf_document = PdfReader(pdf_path)
    output_doc = PdfWriter()

    # 检查起始和结束页码是否有效
    if start_page is not None and end_page is not None and start_page <= end_page:
        # 添加指定范围的页面到新的PDF文档中
        for page_num in range(start_page, end_page + 1):
            output_doc.add_page(pdf_document.pages[page_num])

        # 保存新的PDF文件
        with open(output_pdf_path, 'wb') as f:
            output_doc.write(f)

        print(f"已截取并保存页面从 {start_page + 1} 到 {end_page + 1} 为 {output_pdf_path}")
    else:
        print("提供的页码范围无效。")
    return output_pdf_path


def extract_pages_twice(pdf_path, output_folder, output_suffix):
    common_header = extract_common_header(pdf_path)
    last_begin_index = 0
    begin_pattern = re.compile(r'第[一二三四五六七八九十]+章\s*招标公告|第一卷')
    pdf_document = PdfReader(pdf_path)
    for i, page in enumerate(pdf_document.pages):
        text = page.extract_text()
        if text:
            cleaned_text = clean_page_content(text,common_header)
            # 检查“第一章”开始的位置
            if begin_pattern.search(cleaned_text):
                last_begin_index = i  # 更新最后匹配的索引，页码从0开始
    if output_suffix == "qualification":
        common_pattern = r'^(?:附录(?:一)?[：:]|附件(?:一)?[：:]|附表(?:一)?[：:])'
        # end_pattern = r'^(第[一二三四五六七八九十]+章\s*投标人须知|评标办法|评标办法前附表)'
        end_pattern = re.compile(
            common_pattern + r'(?!.*(?:资质|能力|信誉)).*$|'  # 排除资质、能力、信誉的描述
                             r'^(第[一二三四五六七八九十]+章\s*评标办法|评标办法前附表|投标人须知)',  # 新增的匹配项
            re.MULTILINE
        )
        start_page = None
        end_page = None

        # 从章节开始后的位置进行检查
        for i, page in enumerate(pdf_document.pages[last_begin_index:], start=last_begin_index):
            text = page.extract_text()
            if text:
                cleaned_text = clean_page_content(text,common_header)
                # 确定起始页，需在last_begin_index之后
                if ("资格审查" in cleaned_text or "资质条件" in cleaned_text):
                    if re.search(common_pattern, cleaned_text, re.MULTILINE):
                        if start_page is None:
                            start_page = i  # 确保起始页不小于章节的开始页码
                # 确定结束页
                if start_page is not None and re.search(end_pattern, cleaned_text):
                    if i > start_page:
                        end_page = i
                        break  # 找到结束页后退出循环

        if start_page is None or end_page is None:
            print(f"未找到起始或结束页在文件 {pdf_path} 中！")
            return ""
        else:
            return save_pages_to_new_pdf(pdf_path, output_folder, output_suffix, start_page, end_page)
    elif output_suffix == "invalid":
        pdf_document = PdfReader(pdf_path)
        total_pages = len(pdf_document.pages)
        # 计算总页数的三分之二
        total = int(total_pages * 2 / 3)
        start_page = last_begin_index
        end_page = min(90, total)
        return save_pages_to_new_pdf(pdf_path, output_folder, output_suffix, start_page, end_page)


def extract_pages(pdf_path, output_folder, begin_pattern, begin_page, end_pattern, output_suffix):
    common_header=extract_common_header(pdf_path)
    # 打开PDF文件
    pdf_document = PdfReader(pdf_path)
    start_page = None
    end_page = None
    # 遍历文档的每一页，查找开始和结束短语的位置
    for i in range(len(pdf_document.pages)):
        page = pdf_document.pages[i]
        text = page.extract_text()
        if text:
            cleaned_text = clean_page_content(text,common_header)
            # print(cleaned_text)
            if re.search(begin_pattern, cleaned_text) and i > begin_page:
                if output_suffix == "invalid" and start_page:  # 仅当提取Invalid的时候，判断初始页码是第一个匹配到的页码，因为招标编号可能存在多个，后面的覆盖前面
                    continue
                else:
                    start_page = i
            if start_page is not None and re.search(end_pattern, cleaned_text):
                # 如果output_suffix是"qualification"，调整条件检查
                if output_suffix == "qualification":
                    condition = i > start_page
                else:
                    condition = i > (start_page + 1)
                if condition:
                    is_invalid_condition = output_suffix == "invalid" and i > 30  # 这边默认无效投标至少有30页
                    if is_invalid_condition or output_suffix != "invalid":
                        end_page = i
                        break

    # 确保找到了起始和结束页面
    if start_page is None or end_page is None:
        if output_suffix == "qualification" or output_suffix =="invalid":
            extract_pages_twice(pdf_path, output_folder, output_suffix)
        else:
            print(f"未找到起始或结束页在文件 {pdf_path} 中！")
            return ""
    else:
        return save_pages_to_new_pdf(pdf_path, output_folder, output_suffix, start_page, end_page)


def process_input(input_path, output_folder, begin_pattern, begin_page, end_pattern, output_suffix):
    # 确保输出文件夹存在
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)
    if os.path.isdir(input_path):
        generated_files = []
        # 遍历文件夹内的所有PDF文件
        for file in os.listdir(input_path):
            if file.endswith(".pdf"):
                pdf_path = os.path.join(input_path, file)
                output_pdf_path = extract_pages(pdf_path, output_folder, begin_pattern, begin_page, end_pattern,
                                                output_suffix)
                if output_pdf_path and os.path.isfile(output_pdf_path):
                    generated_files.append(output_pdf_path)
        return generated_files
    elif os.path.isfile(input_path) and input_path.endswith(".pdf"):
        # 处理单个PDF文件
        output_pdf_path = extract_pages(input_path, output_folder, begin_pattern, begin_page, end_pattern,
                                        output_suffix)
        if output_pdf_path and os.path.isfile(output_pdf_path):
            return [output_pdf_path]  # 以列表形式返回，以保持一致性
    else:
        print("提供的路径既不是文件夹也不是PDF文件。")
    return []


def truncate_pdf_main(input_path, output_folder, selection):
    if selection == 1:
        # Configure patterns and phrases for "投标人须知前附表"
        begin_pattern = re.compile(r'第[一二三四五六七八九十]+章\s*投标人须知')
        begin_page = 3
        end_pattern = re.compile(r'投标人须知正文')
        output_suffix = "tobidders_notice_table"
    elif selection == 2:
        # Configure patterns and phrases for "评标办法"
        begin_pattern = re.compile(
            r'第[一二三四五六七八九十]+章\s*评标办法')  # 考虑到这种情况 '第三章 第三章 第三章 第三章     评标办法 评标办法 评标办法 评标办法'
        begin_page = 10
        end_pattern = re.compile(r'评标办法正文|评标办法')
        output_suffix = "evaluation_method"
    elif selection == 3:
        # Configure patterns and phrases for "投标人须知正文"
        begin_pattern = re.compile(r'投标人须知正文')
        begin_page = 5
        end_pattern = re.compile(
            r'^第[一二三四五六七八九十]+章\s*评标办法|^评标办法前附表|^附录(?:一)?[：:]|^附件(?:一)?[：:]|^附表(?:一)?[：:]',
            re.MULTILINE)
        output_suffix = "tobidders_notice"
    elif selection == 4:
        # 配置用于 "资格审查条件" 的正则表达式模式和短语
        common_pattern = r'^(?:附录(?:一)?[：:]|附件(?:一)?[：:]|附表(?:一)?[：:])'
        begin_pattern = re.compile(common_pattern + r'.*(?:资质|能力|信誉).*$', re.MULTILINE)
        begin_page = 5
        end_pattern = re.compile(
            common_pattern + r'(?!.*(?:资质|能力|信誉)).*$|'  # 原有的模式
                             r'^(第[一二三四五六七八九十]+章\s*评标办法|评标办法前附表)',  # 新增的匹配项
            re.MULTILINE
        )
        output_suffix = "qualification"
    elif selection == 5:
        # 配置用于 "招标公告" 的正则表达式模式和短语
        begin_pattern = re.compile(r'第[一二三四五六七八九十]+章\s*招标公告|第一卷')
        begin_page = 0
        end_pattern = re.compile(r'第[一二三四五六七八九十]+章\s*投标人须知', re.MULTILINE)
        output_suffix = "notice"
    elif selection == 6:
        # 配置用于 "无效标" 的正则表达式模式和短语
        begin_pattern = re.compile(r'第[一二三四五六七八九十]+章\s*招标公告|第一卷|招标编号:|招标编号：')
        begin_page = 0
        end_pattern = re.compile(r'第[一二三四五六七八九十]+章\s*合同|[：:]清标报告|第二卷', re.MULTILINE)
        output_suffix = "invalid"
    else:
        print("无效的选择")
        return None

    # Process the selected input
    return process_input(input_path, output_folder, begin_pattern, begin_page, end_pattern, output_suffix)


def truncate_pdf_multiple(input_path, output_folder):
    truncate_files = []
    for selection in range(1, 5):
        files = truncate_pdf_main(input_path, output_folder, selection)
        truncate_files.extend(files)
    return truncate_files


# TODO:需要完善二次请求。目前invalid一定能返回 前附表  须知正文如果为空的话要额外处理一下，比如说就不进行跳转（见xx表） 开评定标这里也要考虑  如果评分表为空，也要处理。
if __name__ == "__main__":
    input_path = "C:\\Users\\Administrator\\Desktop\\招标文件\\test\\zbtest18.pdf"
    output_folder = "C:\\Users\\Administrator\\Desktop\\招标文件\\test"
    # truncate_pdf_multiple(input_path,output_folder)
    selection = 5  # 例如：1 - 投标人须知前附表, 2 - 评标办法, 3 - 投标人须知正文 4-资格审查条件 5-招标公告 6-无效标
    generated_files = truncate_pdf_main(input_path, output_folder, selection)
    # # print("生成的文件:", generated_files)
